//=============================================================================
// (c) Copyright 2015 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEDAQ0804ADInt.c  v7.00
//
// Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
// Description: This sample code demonstrates the usage of UD function
//				DAQ0804ADInt().Performs A/D conversions using interrupt-based I/O with 
//				one A/D conversion per A/D clock tick. The interrupt operation is triggered by the onboard FPGA which raises 
//				an interrupt to the driver after the desired number of conversions take place and 
//				the samples are stored in the FIFO buffer. 
//
//=============================================================================
#include "DSCUD_demo_def.h"
#include "mpedaq0804.h"



// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCBP dsccbp; // structure containing board settings for PCI Express and FeaturePak boards
DAQ0804ADSETTINGS dscadsettings; // structure containing A/D conversion settings
DAQ0804ADINT	 dscIntSettings;//  structure containing A/D conversion interrupt settings
DAQ0804ADINTSTATUS intstatus;// structure containing A/D conversion interrupt status
int sample[8];       // sample reading
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variable of size int
DFLOAT voltage;
int iindex,scansize; // miscellaneous counter
float rate;

char input_buffer[20]; // Buffer for parsing user character input.
int channel=0;

BoardInfo *bi;
DAQ0804COUNTER Ctr ;
#define MPEDAQ_0804_MAX_AD_CHANNEL_NUMBER	7

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. SAMPLING USING AD INTERRUPT
//	    IV. Cleanup
//
//=============================================================================

int main()
{
	int Vrefs[2]={5,10};
	int VrefsBip[2]={10,20};
	double outputVoltage = 0.0;
	int key = 0;
	int Pause = 0;
		
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		printf ("dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the MPEDAQ0804 board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================


	dsccbp.boardtype = DSC_MPEDAQ0804;

	if ( DAQ0804InitBoard ( &dsccbp) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		printf("DAQ0804InitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}
	bi=DSCGetBoardInfo(dsccbp.boardnum);
	printf("\nMPE-DAQ0804 AD INTERRUPT FUNCTION DEMO\n");
	//================================================================================================================
	// III. SAMPLING USING AD INTERRUPT
	//
	//     DAQ0804ADSetSettings() function gets the channel,range,input mode,polarity,clock source from the users
	//	   and configures the board,DAQ0804ADInt() gets the inputs from the user to configure the FIFO settings,
	//     DAQ0804ADIntStatus() function displays the number of conversion completed per second ,DAQ0804ADIntPause()
	//     function stops the conversion with a space bar stroke from the user,DAQ0804ADIntResume() function
	//	   resumes the conversion with a space bar stroke from the user.
	//================================================================================================================
	
	do
	{
		memset(&dscadsettings,0,sizeof(DAQ0804ADSETTINGS));
		memset(&dscIntSettings, 0, sizeof(DAQ0804ADINT));
		memset(&intstatus, 0, sizeof(DAQ0804ADINTSTATUS));

		
		printf ( "Enter low channel value (0-%d, default:0): ", MPEDAQ_0804_MAX_AD_CHANNEL_NUMBER );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Lowch = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Lowch );
			 
		}
		
		printf ( "Enter High channel value(0-%d, default:7): ", MPEDAQ_0804_MAX_AD_CHANNEL_NUMBER );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Highch = 7;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Highch );
			 
		}
		scansize = dscadsettings.Highch - dscadsettings.Lowch +1;
		printf ( "Enter A/D polarity (0 for bipolar, 1 for unipolar, default: 0): " );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Polarity = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Polarity );
			 
		}

		printf("Enter A/D full scale value (0 = 5V, 1 = 10V; default 0):");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Range = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Range );
			
		}
		
		printf("Enter input mode (0 = single-ended, 1 = differential; default 0):");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Sedi = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Sedi );
			
		}
		dscadsettings.ScanEnable = 0;		
		if(dscadsettings.Lowch != dscadsettings.Highch)
		{
			dscadsettings.ScanEnable = 1;		
			printf("Enter Scan Interval (0 = 10us, 1 = 12.5us, 2 = 20us, 3 = programmable, default 0):");
			fgets ( input_buffer, 20, stdin );
			if ( input_buffer[0] == '\n' )
			{
				dscadsettings.ScanInterval = 0;
			}
			else
			{
				sscanf ( input_buffer, "%d", &dscadsettings.ScanInterval );
				 
			}
		
			if(dscadsettings.ScanInterval == 3)
			{
				printf("Enter Programmable Scan Interval value (interval is this value times 100ns)(100-255, default 100):");
				fgets ( input_buffer, 20, stdin );
				if ( input_buffer[0] == '\n' )
				{
					dscadsettings.ProgInt = 0;
				}
				else
				{
					sscanf ( input_buffer, "%d", &dscadsettings.ProgInt );
					 
				}
			}
		}
		
		
		printf("Enter A/D clock source (1-3, 1= External trigger 2=Counter 0, 3=Counter 1 default=2 ) :");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.ADClock = 2;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.ADClock );			 
		}
		
		if(dscadsettings.ADClock !=1)
		{
			printf("Enter A/D sampling Rate (Default 1000) :" );
			fgets ( input_buffer, 20, stdin );
			if ( input_buffer[0] == '\n' )
			{
				rate =1000.0;
			}
			else
			{
				sscanf ( input_buffer, "%f", &rate );			 
			}
		}
		
		if ( ( DAQ0804ADSetSettings ( bi, &dscadsettings ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			printf ("DAQ0804ADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}
		if ( (DAQ0804ADSetTiming(bi,&dscadsettings) ) != DE_NONE )
		{
				dscGetLastError(&errorParams);
				printf ( "DAQ0804ADSetTiming error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
		}
		

		printf ("Enter number of A/D conversions (must be multiple of FIFO threshold value default 1000): " );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscIntSettings.NumConversions = 1000;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscIntSettings.NumConversions );
		}
		
		printf ( "Enter the cycle flag (0 = one shot operation, 1 = continuous operation, default: 1) : " );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscIntSettings.Cycle = 1;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscIntSettings.Cycle );
		}
		
		printf ( "Enter FIFO Enable bit value (0 = disable, 1 = enable, default: 1) : " );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscIntSettings.FIFOEnable = 1;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscIntSettings.FIFOEnable );
		}
		
		if(dscIntSettings.FIFOEnable)
		{
			
			printf ( "Enter FIFO Threshold value (0-2048  default: 1000) : " );
			fgets ( input_buffer, 20, stdin );
			if ( input_buffer[0] == '\n' )
			{
				dscIntSettings.FIFOThreshold = 1000;
			}
			else
			{
				sscanf ( input_buffer, "%d", &dscIntSettings.FIFOThreshold );
			}
			
		}
		
		
		dscIntSettings.ADBuffer = (SWORD*) malloc (sizeof(SWORD)* dscIntSettings.NumConversions );
		
		if(dscIntSettings.ADBuffer == NULL)
		{
		   printf("Could not allocate memory for ADBuffer \n");
		   return 0;
		}
		if( DAQ0804ADInt(bi,&dscIntSettings ) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf ( "DAQ0804ADInt error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}
		if(dscadsettings.ADClock !=1)
		{
			Ctr.Rate = rate ;
			Ctr.CtrNo = dscadsettings.ADClock-2 ;
			Ctr.CtrOutEn 	= 0 ;
			Ctr.CtrOutPol 	= 0 ;
			Ctr.ctrOutWidth = 0 ;
						
			if( DAQ0804CounterSetRate(bi,&Ctr) !=DE_NONE)
			{
				dscGetLastError(&errorParams);
				printf ( "DAQ0804ADInt error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}
		}
		
		if ( (DAQ0804ADSetTiming(bi,&dscadsettings) ) != DE_NONE )
		{
				dscGetLastError(&errorParams);
				printf ( "DAQ0804ADSetTiming error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
		}
		
		dscSleep(1000);
		printf("\nPress Space bar key to Pause/Resume A/D Int or Press ENTER key to stop A/D Interrupt \n");
		
		while( 1 )
		{
		    key = kbhit();
		    if(key==0)
		    {
			if(DAQ0804ADIntStatus(bi,&intstatus) !=DE_NONE)
				{
					dscGetLastError(&errorParams);
					printf ( "DAQ0804ADIntStatus error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
					return 0;
				}
			
			
				printf("No of A/D conversions completed %d\n",intstatus.NumConversions);
													
				dscSleep(1000);
				if( (dscIntSettings.Cycle == 0) && (intstatus.NumConversions >=dscIntSettings.NumConversions) ) break;
		    }
		    else
		    {
			 key = getch();
			 if(key ==32 )
			{
				if(Pause )
				{
					
					if(DAQ0804ADIntResume(bi) !=DE_NONE)
					{
						dscGetLastError(&errorParams);
						printf ( "DAQ0804ADIntResume error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
						return 0;
					}
					Pause = 0;
				}
				else
				{
					
					if(DAQ0804ADIntPause(bi) !=DE_NONE)
					{
						dscGetLastError(&errorParams);
						printf ( "DAQ0804ADIntPause error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
						return 0;
					}

					Pause = 1 ;
				}
			}
			else
			{
				break;
			}
		    }
					
			
					
			
		}			
			
			printf("Channel Num\t");
			for(iindex= dscadsettings.Lowch; iindex <= dscadsettings.Highch; iindex++)
			{
				printf("%d\t",iindex);							
			}
			printf("\n");
			printf("A/DValue(Dec)\t");
			for(iindex= 0; iindex <scansize; iindex++)
			{
				if(dscadsettings.Polarity == 0)
				{
					printf("%d\t",(signed short)dscIntSettings.ADBuffer[iindex]);
				}
				else
				{
					printf("%d\t",(unsigned short)dscIntSettings.ADBuffer[iindex]);
				}
			}
			printf("\n");
			printf("A/DValue(Hex)\t");

			for(iindex= 0; iindex <scansize; iindex++)
			{
				
					printf("0x%X\t",(unsigned short)dscIntSettings.ADBuffer[iindex]);
				
			}
			printf("\n");
			printf("Volts   \t");
			 for(iindex= 0; iindex <scansize; iindex++)
			{ 
            
				if(dscadsettings.Polarity == 0)
				{
			
					outputVoltage =	(double)((signed short) dscIntSettings.ADBuffer[iindex]/65535.0)* VrefsBip[dscadsettings.Range];
		
				}
				else
				{
					outputVoltage =	(double)((unsigned short)dscIntSettings.ADBuffer[iindex]/65535.0)* Vrefs[dscadsettings.Range];
				}
				printf("%0.4lf\t",outputVoltage);
			}
			printf("\n\n");
		printf("Press Enter to repeat or 'q' to quit \n");
		fgets(input_buffer,20,stdin);
		
		DAQ0804ADIntCancel(bi);
		free(dscIntSettings.ADBuffer);
		}while(input_buffer[0] !='q');
	//=========================================================================
	// V. CLEANUP
	//
	//	  Cleanup any remnants left by the program and free the resources used
	//	  by the driver.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. free the driver resources
	//====================================================================

	dscFree();

	

	printf ( "\nDSC A/D interrupt completed.\n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
