//=============================================================================
// (c) Copyright 2015 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEDAQ0804ADScan.c   v7.00
//
// Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
// Description: This sample code demonstrates the usage of UD function
//				dscADScan(). This sample code describes the usage
//				for MPEDAQ0804. Hardware on the board will be commanded
//              to scan the A/D channels passed, and return them to the
//              sample array. Then the values will be interpreted based on
//              the mode supplied by the user.
//              Each A/D hardware scan consists of a sweep of all the
//              selected channels at a time, rather than only one channel.
//
//=============================================================================

#include "DSCUD_demo_def.h"
#include "mpedaq0804.h"



// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCBP dsccbp; // structure containing board settings for PCI Express and FeaturePak boards
DAQ0804ADSETTINGS dscadsettings; // structure containing A/D conversion settings
unsigned int sample[8];       // sample reading
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variable of size int
DFLOAT voltage;
int iindex,scansize;	// miscellaneous counter

char input_buffer[20]; // Buffer for parsing user character input.
int channel=0;

BoardInfo *bi;
#define MPEDAQ_0804_MAX_AD_CHANNEL_NUMBER	7

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. SAMPLING AND OUTPUT
//	    VI. Cleanup
//
//=============================================================================

int main()
{
	int Vrefs[2]={5,10};
	int VrefsBip[2]={10,20};
	double outputVoltage = 0.0;
	memset(&dscadsettings,0,sizeof(DAQ0804ADSETTINGS));
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		printf ("dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the MPEDAQ0804DAQ board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================


	dsccbp.boardtype = DSC_MPEDAQ0804;

	if ( DAQ0804InitBoard ( &dsccbp) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		printf("DAQ0804InitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}
	bi=DSCGetBoardInfo(dsccbp.boardnum);
	printf("\n MPE-DAQ0804 AD SCAN FUNCTION DEMO\n");
	//================================================================================================================
	// III. SAMPLING AND OUTPUT
	//
	//     DAQ0804ADSetSettings() function gets the Lower channel and Higher channel,range,input mode,polarity,clock 
	//	   source from the user and configures the board.DAQ0804ADTrigger function() converts the sample,displays the  
	//	   converted value for the channels in the Lower channel to Higher channel range.
	//================================================================================================================
	do
	{
		printf ( "Enter the low channel(0-%d, default:0): ", MPEDAQ_0804_MAX_AD_CHANNEL_NUMBER );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Lowch=0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Lowch );
			 
		}
		
		printf ( "Enter the High channel (0-%d, default:7): ", MPEDAQ_0804_MAX_AD_CHANNEL_NUMBER );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Highch=7;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Highch );
			 
		}
		scansize = dscadsettings.Highch - dscadsettings.Lowch + 1;
		printf ( "Enter A/D polarity (0 for bipolar, 1 for unipolar, default: 0): " );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Polarity = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Polarity );
			 
		}

		printf("Enter A/D full scale value (0 = 5V, 1 = 10V; default 0):");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Range = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Range );
			
		}
		
		printf("Enter input mode (0 = single-ended, 1 = differential; default 0):");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Sedi = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Sedi );
			
		}
		
		printf("Enter Scan Interval (0 = 10us, 1 = 12.5us, 2 = 20us, 3 = programmable, default 0):");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.ScanInterval = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.ScanInterval );
			 
		}
		
		if(dscadsettings.ScanInterval == 3)
		{
			printf("Enter Programmable Scan Interval value (interval is this value times 100ns)(100-255, default 100):");
			fgets ( input_buffer, 20, stdin );
			if ( input_buffer[0] == '\n' )
			{
				dscadsettings.ProgInt = 0;
			}
			else
			{
				sscanf ( input_buffer, "%d", &dscadsettings.ProgInt );
				 
			}
		}
		
				
		dscadsettings.ADClock = 0;
		dscadsettings.ScanEnable = 1;		
		
		if ( ( DAQ0804ADSetSettings ( bi, &dscadsettings ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			printf ("DAQ0804ADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}
		if ( (DAQ0804ADSetTiming(bi,&dscadsettings) ) != DE_NONE )
		{
				dscGetLastError(&errorParams);
				printf ( "DAQ0804ADSetTiming error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
		}
		
		printf("Press ENTER key to stop scanning A/D channel \n");
		while(!kbhit() )
		{
			if ( (DAQ0804ADTrigger ( bi, sample ) ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				printf ( "DAQ0804ADTrigger error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}
			printf("Channel Num\t");
			for(iindex= dscadsettings.Lowch; iindex <= dscadsettings.Highch; iindex++)
			{
				printf("%d\t",iindex);							
			}
			printf("\n");
			printf("A/DValue(Dec)\t");
			for(iindex= 0; iindex <scansize; iindex++)
			{
				if(dscadsettings.Polarity == 1)
					printf("%d\t",sample[iindex]);
				else
					printf("%d\t",(signed short)sample[iindex]);
			}
			printf("\n");
			printf("A/DValue(Hex)\t");

			for(iindex= 0; iindex <scansize; iindex++)
			{
				printf("0x%X\t",sample[iindex]);
			}
			printf("\n");
			printf("Volts   \t");
			 for(iindex= 0; iindex <scansize; iindex++)
			{ 
            
				if(dscadsettings.Polarity == 0)
				{
			
			outputVoltage =	(double)((signed short) sample[iindex]/65535.0)* VrefsBip[dscadsettings.Range];
		
				}
				else
				{
				outputVoltage =	(double)(sample[iindex]/65535.0)* Vrefs[dscadsettings.Range];
				}
				printf("%0.4lf\t",outputVoltage);
			}
			printf("\n\n");
			dscSleep(1000);
		}
		
		fgets ( input_buffer, 20, stdin );
		
		printf("Press Enter to repeat or 'q' to quit \n");
		fgets(input_buffer,20,stdin);
	}while(input_buffer[0] !='q');
	//=========================================================================
	// V. CLEANUP
	//
	//	  Cleanup any remnants left by the program and free the resources used
	//	  by the driver.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. free the driver resources
	//====================================================================

	dscFree();

	

	printf ( "\nDSCADScan completed.\n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
