//=============================================================================
// (c) Copyright 2015 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEDAQ0804ADTrigger.c   v3.00
//
// Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
// Description: This sample code demonstrates the usage of UD function
//				DAQ0804ADTrigger(). This sample code describes the usage
//				for MPEDAQ0804. An A/D Sample will be taken from the
//              onboard hardware, and transferred to a value in user space;
//              this value will then be interpreted based on the mode
//              that the user set.
//
//=============================================================================
#include "DSCUD_demo_def.h"
#include "mpedaq0804.h"



// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCBP dsccbp; // structure containing board settings for PCI Express and FeaturePak boards
DAQ0804ADSETTINGS dscadsettings; // structure containing A/D conversion settings
DAQ0804ADINT	 dscIntSettings;//  structure containing A/D conversion interrupt settings
DAQ0804ADINTSTATUS intstatus;//structure containing A/D conversion interrupt status
int sample;       // sample reading
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variable of size int
DFLOAT voltage;
int i;	// miscellaneous counter

char input_buffer[20]; // Buffer for parsing user character input.
int channel=0;

BoardInfo *bi;

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. SAMPLING AND OUTPUT
//	    VI. Cleanup
//
//=============================================================================

int main()
{
	int Vrefs[2]={5,10};
	int VrefsBip[2]={10,20};
	double outputVoltage = 0.0;
	int key = 0 , Pause = 0;
	memset(&dscadsettings,0,sizeof(DAQ0804ADSETTINGS));
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		printf ("dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the MPEDAQ0804 board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	dsccbp.boardtype = DSC_MPEDAQ0804;

	if ( DAQ0804InitBoard ( &dsccbp) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		printf("DAQ0804InitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}
    	bi=DSCGetBoardInfo(dsccbp.boardnum);
     	printf("\nMPEDAQ0804 AD TRIGGER FUNCTION DEMO\n");
	//================================================================================================================
	// III. SAMPLING AND OUTPUT
	//
	//     DAQ0804ADSetSettings() function gets the channel,range,input mode,polarity,clock source from the users
	//	   and configures the board.If the Clock source is chosen as Manual, DAQ0804ADTrigger() function converts the 
	//	   sample, displays the converted value and the application is exited.If the Clock source is not chosen as 
	//	   Manual DAQ0804ADInt() gets the inputs from the user to configure the FIFO settings,DAQ0804ADIntStatus() 
	//     function displays the number of conversion completed per second ,DAQ0804ADIntPause() function stops the 
	//     conversion with a space bar stroke from the user,DAQ0804ADIntResume() function resumes the conversion with 
	//	   a space bar stroke from the user.
	//================================================================================================================

	do
	{
		
		printf("Enter channel number(0-7,default:0):");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			channel=0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &channel );
			 
		}
		
		printf ( "Enter A/D polarity (0 for bipolar, 1 for unipolar, default: 0):" );
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Polarity = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Polarity );
			 
		}

		printf("Enter A/D full scale value (0 = 5V, 1 = 10V; default 0):");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Range = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Range );
			
		}
		
		printf("Enter input mode (0 = single-ended, 1 = differential; default 0):");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.Sedi = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.Sedi );
			
		}
				
		dscadsettings.ScanEnable = 0;	
		dscadsettings.Highch = channel;
		dscadsettings.Lowch = channel;
		printf("Enter Clock source (0 = Manual, 1 = falling edge of external trigger (DIO bit 19), default 0):");
		fgets ( input_buffer, 20, stdin );
		if ( input_buffer[0] == '\n' )
		{
			dscadsettings.ADClock = 0;
		}
		else
		{
			sscanf ( input_buffer, "%d", &dscadsettings.ADClock );
			
		}
		if ( ( DAQ0804ADSetSettings ( bi, &dscadsettings ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			printf ("DAQ0804ADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}
				
		if ( (DAQ0804ADSetTiming(bi,&dscadsettings) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			printf ( "DAQ0804ADSetChannel error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}

		if(dscadsettings.ADClock == 0)
		{
			if ( (DAQ0804ADTrigger ( bi, &sample ) ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				printf ( "DAQ0804ADTrigger error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}
				 
							
			if(dscadsettings.Polarity == 0)
			{
				
				outputVoltage =	(double)((signed short) sample/65535.0)* VrefsBip[dscadsettings.Range];
				printf("%d\t0x%X\t%0.4lf\n",(signed short)sample,sample,outputVoltage);
			
			}
			else
			{
				outputVoltage =	(double)(sample/65535.0)* Vrefs[dscadsettings.Range];
				printf("%d\t0x%X\t%0.4lf\n",sample,sample,outputVoltage);
			}
		}
		else
		{
			printf ("Enter number of A/D conversions (must be multiple of FIFO threshold value default 1000): " );
			fgets ( input_buffer, 20, stdin );
			if ( input_buffer[0] == '\n' )
			{
				dscIntSettings.NumConversions = 1000;
			}
			else
			{
				sscanf ( input_buffer, "%d", &dscIntSettings.NumConversions );
			}
			
			printf ( "Enter the cycle flag (0 = one shot operation, 1 = continuous operation, default: 1) : " );
			fgets ( input_buffer, 20, stdin );
			if ( input_buffer[0] == '\n' )
			{
				dscIntSettings.Cycle = 1;
			}
			else
			{
				sscanf ( input_buffer, "%d", &dscIntSettings.Cycle );
			}
			
			printf ( "Enter FIFO Enable bit value (0 = disable, 1 = enable, default: 1) : " );
			fgets ( input_buffer, 20, stdin );
			if ( input_buffer[0] == '\n' )
			{
				dscIntSettings.FIFOEnable = 1;
			}
			else
			{
				sscanf ( input_buffer, "%d", &dscIntSettings.FIFOEnable );
			}
			
			if(dscIntSettings.FIFOEnable)
			{
				
				printf ( "Enter FIFO Threshold value (0-2048  default: 1000) : " );
				fgets ( input_buffer, 20, stdin );
				if ( input_buffer[0] == '\n' )
				{
					dscIntSettings.FIFOThreshold = 1000;
				}
				else
				{
					sscanf ( input_buffer, "%d", &dscIntSettings.FIFOThreshold );
				}
				
			}
			
			
			dscIntSettings.ADBuffer = (SWORD*) malloc (sizeof(SWORD)* dscIntSettings.NumConversions );
			
			if(dscIntSettings.ADBuffer == NULL)
			{
			   printf("Could not allocate memory for ADBuffer \n");
			   return 0;
			}
			if( DAQ0804ADInt(bi,&dscIntSettings ) !=DE_NONE)
			{
				dscGetLastError(&errorParams);
				printf ( "DAQ0804ADInt error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}
			
			if ( (DAQ0804ADSetTiming(bi,&dscadsettings) ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				printf ( "DAQ0804ADSetChannel error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}
			
			dscSleep(1000);
		printf("\nPress Space bar key to Pause/Resume A/D Int or Press ENTER key to stop A/D Interrupt \n");
		
		while( 1 )
		{
		    key = kbhit();
		    if(key==0)
		    {
			if(DAQ0804ADIntStatus(bi,&intstatus) !=DE_NONE)
				{
					dscGetLastError(&errorParams);
					printf ( "DAQ0804ADIntStatus error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
					return 0;
				}
			
			
				printf("No of A/D conversions completed %d\n",intstatus.NumConversions);
													
				dscSleep(1000);
				if( (dscIntSettings.Cycle == 0) && (intstatus.NumConversions >=dscIntSettings.NumConversions) ) break;
		    }
		    else
		    {
			 key = getchar();
			 if(key ==32 )
			{
				if(Pause )
				{
					
					if(DAQ0804ADIntResume(bi) !=DE_NONE)
					{
						dscGetLastError(&errorParams);
						printf ( "DAQ0804ADIntResume error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
						return 0;
					}
					Pause = 0;
				}
				else
				{
					
					if(DAQ0804ADIntPause(bi) !=DE_NONE)
					{
						dscGetLastError(&errorParams);
						printf ( "DAQ0804ADIntPause error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
						return 0;
					}
					Pause = 1 ;
				}
			}
			else
			{
				break;
			}
		    }
					
			
					
			
		}
			printf("Channel Num\t%d\n",dscadsettings.Lowch);
					
			printf("A/DValue(Dec)\t");
			
			if(dscadsettings.Polarity == 0)
			{
				printf("%d\n",(signed short)dscIntSettings.ADBuffer[0]);
			}
			else
			{
				printf("%d\n",(unsigned short)dscIntSettings.ADBuffer[0]);
			}
					
			printf("A/DValue(Hex)\t0x%X\n",(unsigned short)dscIntSettings.ADBuffer[0]);		
						
			printf("Volts   \t");
			  
            
			if(dscadsettings.Polarity == 0)
			{
		
				outputVoltage =	(double)((signed short) dscIntSettings.ADBuffer[0]/65535.0)* VrefsBip[dscadsettings.Range];
	
			}
			else
			{
				outputVoltage =	(double)((unsigned short)dscIntSettings.ADBuffer[0]/65535.0)* Vrefs[dscadsettings.Range];
			}
			printf("%0.4lf\n",outputVoltage);
			
			printf("\n\n");
			DAQ0804ADIntCancel(bi);
			free(dscIntSettings.ADBuffer);
		}
		
   		printf ("press Enter to repeat or 'q' to quit:");
   		fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );


	//=========================================================================
	// V. CLEANUP
	//
	//	  Cleanup any remnants left by the program and free the resources used
	//	  by the driver.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. free the driver resources
	//====================================================================

	dscFree();

	

	printf ( "\nDSCADSample completed.\n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
