//=============================================================================
// (c) Copyright 2015 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEDAQ0804CounterSetRate .c  v3.00
//
// Depedency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				
// Description: This sample code demonstrates the usage of UD function  
//				DAQ0804CounterSetRate(). 
//============================================================================

#include "DSCUD_demo_def.h"
#include "mpedaq0804.h"


// globals
ERRPARAMS errorParams; // structure for returning error code and error string
// var declarations
DSCB dscb = 0; // handle used to refer to the board
DSCCB dsccb;    // structure containing ISA board settings
DSCCBP dsccbp; // structure containing PCI board settings
BoardInfo *bi=NULL;
char input_buffer[20];

DAQ0804COUNTER counter;

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. PROGRAM COUNTER
//	    IV. Cleanup
//
//=============================================================================

int main ( void )
{

	int intBuff = 0; // temp variables of size int
	int temp = 0; // temporary storage
	unsigned long ctrData=0;
	float Rate =0;
   	BYTE return_value=0;
	int baseadd = 0;
	memset ( &dsccbp, 0, sizeof(DSCCBP) );
	memset ( &counter, 0 ,sizeof(DAQ0804COUNTER));

	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
		
	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the MPE-DAQ-0804 board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf ( "\nDSC MPE-DAQ-0804 COUNTER RATE GENERATOR DEMO\n" );

	dsccbp.boardtype = DSC_MPEDAQ0804;
	dsccbp.pci_slot  = 0;

	 
	if(DAQ0804InitBoard ( &dsccbp ) !=DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("DAQ0804InitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}
	bi = DSCGetBoardInfo ( dsccbp.boardnum );	
   	
	//=========================================================================
	// III. PROGRAM COUNTER
	//
	//	    DAQ0804CounterSetRate() function configures the Counter circuit in count down direction and
	//	    the counter output is enabled corresponding DIO based on user input, then reads the counter value and displays every second.
	//
	//=========================================================================
	do
	{
		printf("Enter  counter number (0-7)<default:0>:");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			counter.CtrNo = 0;
		}
		else
		{
			sscanf(input_buffer,"%d",&counter.CtrNo);
		}
		
		printf("Enter Counter frequency rate (1-50MHz ) (Default = 1000)");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			counter.Rate = 1000;
		}
		else
		{
			sscanf(input_buffer,"%f",&counter.Rate);
		}
		
		printf("Enter Output Enable (0=disable, 1=Enable; default = 1):");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			counter.CtrOutEn = 1;
		}
		else
		{
			sscanf(input_buffer,"%d",&counter.CtrOutEn);
		}
		
		if(counter.CtrOutEn)
		{
			printf("Enter Output Polarity (0=negative, 1= positive; default = 1):");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0]=='\n')
			{	
				counter.CtrOutPol = 1;
			}
			else
			{
				sscanf(input_buffer,"%d",&counter.CtrOutPol);
			}

			printf("Enter output pulse width (0-3 ,0 = 1 clocks, 1 = 10 clocks, 2 = 100 clocks, 3 = 1000 clocks, default = 3) : ");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0]=='\n')
			{	
				counter.ctrOutWidth = 3;
			}
			else
			{
				sscanf(input_buffer,"%d",&counter.ctrOutWidth);
			}
		}
		
		
		if(DAQ0804CounterSetRate(bi,&counter) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("DAQ0804CounterSetRate error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		printf("Press ENTER key to stop counting \n");
		
		while( !kbhit())
		{
			dscSleep(1000);
			DAQ0804CounterRead(bi,counter.CtrNo,&ctrData);
			printf("Counter Data %ld \r",ctrData);
			fflush(stdout);
			
		}
				
		fgets ( input_buffer, 20, stdin );
		
		if(DAQ0804CounterReset(bi,counter.CtrNo) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("DAQ0804CounterReset error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		printf ("press Enter to repeat or 'q' to quit:");
   		fgets ( input_buffer, 20, stdin );
		
    } while ( input_buffer[0] != 'q' );
	
		
		
	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree ( );
	printf ( "\nDSCCounter Function completed.\n" );
	return 0;

} // end main()

