//=============================================================================
// (c) Copyright 2015 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEDAQ0804DIOFunctions.c  v7.00
//
// Depedency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				
//
// MPEDAQ0804DIOFunctions.c  v7.0
// Desc: Writes and reads from Digital I/O port
//============================================================================
#include "DSCUD_demo_def.h"
#define DIOPORT_MAX 3
//DIO Port No
#define DIOPORT_A 0
#define DIOPORT_B 1
#define DIOPORT_C 2



#define READ_BYTE					   1
#define READ_BYTE_FROM_ALL_PORTS	   2
#define READ_ALL_BITS				   3
#define WRITE_BYTE					   4
#define WRITE_BIT					   5
#define LOOP_BACK_TEST				   6


// globals
ERRPARAMS errorParams; // structure for returning error code and error string
// var declarations
DSCB dscb = 0; // handle used to refer to the board
DSCCBP dsccbp; // structure containing board settings
int portConfig[3];//used to store the port directions


//new variables
#include "mpedaq0804.h"
BoardInfo *bi;
char input_buffer[20];

void readByteDIOPort();
void readByteAllDIOPorts();
void readAllBitsDIOPort();
void writeByteDIOPort();
void writeBitDIOPort();
void DIOPortLoopbackTest();
//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. DIO INPUT AND OUTPUT
//	    IV. Cleanup
//
//=============================================================================

int main ( void )
{

	int intBuff = 0; // temp variables of size int
	int temp = 0; // temporary storage
   	char input_buffer[20];

   	memset ( &dsccbp, 0, sizeof(DSCCBP) );
	memset (portConfig,0, sizeof(int)*DIOPORT_MAX);

  
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the port_under_testboard. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEDAQ0804 for the MPEDAQ0804 board
	//	   2. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf ( "\nDSC MPE-DAQ0804 DIO FUNCTIONS DEMO\n" );

	dsccbp.boardtype = DSC_MPEDAQ0804;
        dsccbp.pci_slot  =0;

	if ( (DAQ0804InitBoard (&dsccbp)!= DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf (  "DAQ0804InitBoard error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
	bi=DSCGetBoardInfo(dsccbp.boardnum);
     printf("The base address of MPE-DAQ0804 board is %lx \n",dsccbp.io_address[0]);
 

    //=========================================================================
	// III. DIO INPUT AND OUTPUT
	//
	//		Perform a set of byte-level inputs and outputs. We start by sending
	//		an output byte value of 0 to the selected digital I/O port and then
	//		read from that same port again to verify that the correct value was
	//		exchanged. For word and bit-level operations, the steps are nearly
	//		identical with a few minor changes to function names and the size
	//		of the data to be transferred.
	//
	//		NOTE: Remember that the bytes received from port are supposed to be
	//			  equal to the inversion of the bytes sent. That is, the input
	//			  bytes are equal to 255 minus the output bytes.
	//
	//		Guide:
	//
	//		1. Read a Byte from  port 
	//		2. Read a Byte from all ports
	//		3. Read all bits from a  port
	//		4. Write a Byte to port
	//		5. Write a Bit to a port
	//		6. Port loopback with 0-255 read/write repetitive\n
	//		q. Quit Program
	// 		Choose these options in any order from the menu!
	//=========================================================================

	printf( "\nDIO INPUT AND OUTPUT: Main Menu \n" );
	do
	{
		intBuff = 0;
		printf (  "########################################################\n" );
		printf (  "1) Read a Byte from  port\n" );
		printf (  "2) Read a Byte from all ports\n" );
		printf (  "3) Read all bits from a  port\n" );
		printf (  "4) Write a Byte to port\n" );
		printf (  "5) Write a Bit to a port\n" );
		printf (  "6) Port loopback with 0-255 read/write repetitive\n" );
		printf (  "q) Quit Program \n" );
		printf (  "########################################################\n" );

		fgets ( input_buffer, 20, stdin );
        sscanf ( input_buffer, "%d", &intBuff );

        if ( input_buffer[0] == 'q' )
        {
            intBuff = 'q';
        }

			switch ( intBuff )
		{
				case READ_BYTE:
					readByteDIOPort();
					break;

				case READ_BYTE_FROM_ALL_PORTS:
	     			readByteAllDIOPorts();
					break;

				case READ_ALL_BITS:
					 readAllBitsDIOPort();
					 break;

				case WRITE_BYTE:
					   writeByteDIOPort();
					   break;
				case WRITE_BIT:
					  writeBitDIOPort();
					  break;

				case LOOP_BACK_TEST:
					DIOPortLoopbackTest();
					break;
				
				case 'q':
					return 0;
            		break;

				default: // invalid option selected
					printf("Please Enter valid option \r\n");
					break;
		}
				
	} while ( input_buffer[0] != 'q' );


	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree ( );

	printf ( "\nDSCDIOFunctions completed.\n" );

	return 0;

} // end main()

//=============================================================================
// Name: readByteDIOPort()
// Desc: Read a port A-C by receiving a byte from the selected port's  register.
// Parameters:void
// Return: void
//=============================================================================
void readByteDIOPort()
{
	int *config;
	int port = 0;
	BYTE input_byte = 0; 
	config = (int *)malloc(sizeof(int)* 3);
	config[0] = 0;
	config[1] = 0;
	config[2] = 0;
	
	DAQ0804DIOConfigAll(bi,config);

	printf ("Enter port number (0-2):");
	fgets ( input_buffer, 20, stdin );
   	sscanf ( input_buffer, "%d", &port );

	printf("\nPress ENTER key to stop reading ...\r\n");
	

	do
	{

		if( (DAQ0804DIOInputByte ( bi, port, &input_byte ) != DE_NONE) )
		{
			dscGetLastError ( &errorParams );
			printf (  "DAQ0804DIOInputByte() error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
			return;
		}

		dscSleep( 1000 );

		printf("Byte value received from port %d = 0x%x\r\n", port, input_byte);

	}
	
	while ( !( kbhit() ) );

	return;

}


//=============================================================================
// Name: readByteAllDIOPorts()
// Desc: Read a BYTE value from all the  ports A-C .
// Parameters:void
// Return: void
//=============================================================================
void readByteAllDIOPorts()
{

	int *config;
	BYTE port = 0;
	BYTE input_byte = 0; 
	config = (int *)malloc(sizeof(int)* 3);
	config[0] = 0;
	config[1] = 0;
	config[2] = 0;
	DAQ0804DIOConfigAll(bi,config);

	printf("\nPress ENTER key to stop reading ...\r\n");
	
	do
	{
		printf("Port:\t0\t1\t2\n");
		printf("Value:\t");
		for(port = 0;port <3; port++)
		{
			if( (DAQ0804DIOInputByte ( bi, port, &input_byte ) != DE_NONE) )
			{
				dscGetLastError ( &errorParams );
				printf (  "DAQ0804DIOInputByte() error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return;
			}
			printf("0x%X\t",input_byte);

		}
		dscSleep(1000);
		printf("\n\n");

	}while ( !( kbhit() ) );

	return;

}

//=============================================================================
// Name: readAllBitsDIOPort()
// Desc: Read all bits  from the selected port's  register.
// Parameters:void
// Return: void
//=============================================================================
void  readAllBitsDIOPort()
{
	int *config;
	int port = 0;
	int digital_value = 0; // digital representation of the bit
	BYTE bit = 0;
	int intBuff = 0; // temp variables of size int
	config = (int *)malloc(sizeof(int)* 3);
	config[0] = 0;
	config[1] = 0;
	config[2] = 0;
	DAQ0804DIOConfigAll(bi,config);	

	printf ("Enter port number (0-2):");
	fgets ( input_buffer, 20, stdin );
    sscanf ( input_buffer, "%d", &port );
	
	printf("\nPress ENTER key to stop reading ...\r\n");

	do
	{
		printf("The port %d each bit values are \n",port);
		printf("--------------------------------------------------------------------\r\n");
		printf("BitNo:\t7\t6\t5\t4\t3\t2\t1\t0\r\n");
		printf("--------------------------------------------------------------------\r\n");
		printf("Value:");
		
		for(intBuff=7 ;intBuff>=0;intBuff--)
		{
			if(port == 0)
			{
				bit = (BYTE) intBuff;
			}
			else if(port == 1)
			{
				bit = 8 + (BYTE) intBuff;
			}
			else if(port == 2)
			{
				bit = 16 + (BYTE) intBuff;
				if(bit >20) 
				{
					printf("\t0");
					continue;
				}
			}
			
			if ( (DAQ0804DIOInputBit ( bi,bit, &digital_value ) != DE_NONE) )
			{
				dscGetLastError ( &errorParams );
				printf("DAQ0804DIOInputBit () error: %s %s\n",
					  dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return;
			}

		printf("\t%x",digital_value);
		
		}

		printf("\n");
		dscSleep(1000);

	}while ( !( kbhit() ) );

	return;


}

//=============================================================================
// Name: writeByteDIOPort()
// Desc: Program a port A-C by sending a byte to the selected port's
//	  register.
// Parameters: void
// Return: void
//=============================================================================
void writeByteDIOPort()
{
	int *config;
	int port = 0;
	BYTE output_byte = 0;
	int intBuff = 0;
	config = (int *)malloc(sizeof(int)* 3);
	config[0] = 0;
	config[1] = 0;
	config[2] = 0;
	DAQ0804DIOConfigAll(bi,config);

	printf ("Enter port number (0-2):");
	fgets ( input_buffer, 20, stdin );
   	sscanf ( input_buffer, "%d", &port );
	
	config[port] = 0xFF;
	DAQ0804DIOConfigAll(bi,config);

	do
	{
		printf("Enter value 0-255 or q to quit:");
		fgets ( input_buffer, 20, stdin );
		sscanf ( input_buffer, "%d", &intBuff );
		
		if ( input_buffer[0] == 'q' )
		{
			intBuff = 'q';
		}
		else
		{
			output_byte = (BYTE)intBuff;
			
			if ( (DAQ0804DIOOutputByte(bi,port,output_byte) != DE_NONE) )
			{
				dscGetLastError ( &errorParams );
				printf("DAQ0804DIOOutputByte () error: %s %s\n",
					  dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return;
			}
			printf("The Byte value  %d is sent to port %d \n",output_byte,port);
		}
	}while ( input_buffer[0] != 'q' );

return;

}

//=============================================================================
// Name: writeBitDIOPort()
// Desc: Program a port A-C by sending a digital value to the selected
// 	  port's bit register.
// Parameters:void
// Return: void
//=============================================================================
void writeBitDIOPort()
{
	int *config;
	BYTE digital_val = 0;
	BYTE bit = 0;
	int port = 0;
	int intBuf = 0;
	config = (int *)malloc(sizeof(int)* 3);
	config[0] = 0;
	config[1] = 0;
	config[2] = 0;
	DAQ0804DIOConfigAll(bi,config);

	printf ("Enter port number (0-2):");
	fgets ( input_buffer, 20, stdin );
    sscanf ( input_buffer, "%d", &port );
	
	config[port] = 0xFF;
	DAQ0804DIOConfigAll(bi,config);
	
	do
	{
		if(port == 2)
			printf ("Enter Bit (0-4):or q to quit :");
		else
			printf ("Enter Bit (0-7):or q to quit :");
		fgets ( input_buffer, 20, stdin );
		sscanf ( input_buffer, "%d", &intBuf );

		if(port == 0)
		{
			bit = intBuf;
		}
		else if(port == 1)
		{
			bit = 8 + intBuf;
		}
		else
		{
			bit = 16 + intBuf;
		}
		if(input_buffer[0] != 'q')
		{
			printf ("Enter Bit value (0-1):");
			fgets ( input_buffer, 20, stdin );
			sscanf ( input_buffer, "%d", &intBuf );
			digital_val = intBuf;


			if ( (DAQ0804DIOOutputBit(bi,bit,digital_val) != DE_NONE) )
			{
				dscGetLastError ( &errorParams );
				printf("DAQ0804DIOOutputBit () error: %s %s\n",
					  dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return;
			}
		}
	}while( input_buffer[0] != 'q' );

return;
}



//=============================================================================
// Name: DIOPortLoopbackTest()
// Desc: loop back test on various ports
// Parameters: void
// Return: void
//=============================================================================
void DIOPortLoopbackTest()
{

	int *config;
	int output_port = 0,input_port = 0;
	BYTE output_byte = 0,input_byte = 0;
	int errorcount=0;
	int intBuff = 0;
	config = (int *)malloc(sizeof(int)* 3);
	config[0] = 0;
	config[1] = 0;
	config[2] = 0;
	DAQ0804DIOConfigAll(bi,config);

	do
	{
		printf ("Enter port number for input (0-2) or  Enter -1 to go to Main menu:");
		fgets ( input_buffer, 20, stdin );
		sscanf ( input_buffer, "%d", &input_port );
		if(input_port==-1) return;
		

		printf ("Enter port number for output (0-2):");
		fgets ( input_buffer, 20, stdin );
		sscanf ( input_buffer, "%d", &output_port );
		if(output_port==-1) return;
			
		if (( input_port > DIOPORT_MAX || input_port < 0 ) || (output_port > DIOPORT_MAX || output_port < 0 ) )
		{
			printf ( "ERROR: Valid Port Numbers are: (0-%d)\n ", DIOPORT_MAX - 1);
			return;
		}

		config[input_port] = 0;
		config[output_port] = 0xFF;
		DAQ0804DIOConfigAll(bi,config);

		errorcount=0;
		for(intBuff=0; intBuff<=255;intBuff++)
		{  
		  output_byte=(BYTE) intBuff;
			
			if( (DAQ0804DIOOutputByte( bi, output_port, output_byte ) != DE_NONE) )
			{
				dscGetLastError ( &errorParams );
				printf (  "DAQ0804DIOOutputByte() error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return;
			}
			
			if( (DAQ0804DIOInputByte ( bi, input_port, &input_byte ) != DE_NONE) )
			{
					dscGetLastError ( &errorParams );
					printf (   "DAQ0804DIOInputByte() error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
					return;
			}

			 
			if(output_byte !=input_byte)
			{
				  printf("Error: Output: 0x%x  Input: 0x%x \n",output_byte,input_byte);
					  errorcount+=1;
			}
		}
		printf("\nValues written : %d and Errors: %d \n",intBuff,errorcount);
	}while(1);
return;
}
